@@ -10,7 +10,7 @@
  *******************************************************************************/
 /*eslint-env node */
 /*eslint no-console:1*/
-var api = require('../api'), writeError = api.writeError;
+var api = require('../api'), writeError = api.writeError, writeResponse = api.writeResponse;
 var git = require('nodegit');
 var url = require("url");
 var path = require("path");
@@ -22,14 +22,21 @@ var tasks = require('../tasks');
 var express = require('express');
 var bodyParser = require('body-parser');
 var rmdir = require('rimraf');
+var credentialsProvider = require('./credentials');
 
 module.exports = {};
 
 module.exports.router = function(options) {
 	var fileRoot = options.fileRoot;
-	if (!fileRoot) { throw new Error('options.root is required'); }
-	var contextPath = options.options.configParams["orion.context.path"] || "";
-
+	var gitRoot = options.gitRoot;
+	var workspaceRoot = options.workspaceRoot;
+	if (!fileRoot) { throw new Error('options.fileRoot is required'); }
+	if (!gitRoot) { throw new Error('options.gitRoot is required'); }
+	if (!workspaceRoot) { throw new Error('options.workspaceRoot is required'); }
+	
+	/* Note that context path was not included in file and workspace root. */
+	var contextPath = options && options.options && options.options.configParams["orion.context.path"] || "";
+	
 	module.exports.getRepo = getRepo;
 	module.exports.getClones = getClones;
 	module.exports.getRemoteCallbacks = getRemoteCallbacks;
@@ -40,42 +47,42 @@ module.exports.router = function(options) {
 	module.exports.getfileDirPath = getfileDirPath;
 	module.exports.getfileAbsolutePath = getfileAbsolutePath;
 	module.exports.getfileRelativePath = getfileRelativePath;
-	module.exports.isWorkspace = isWorkspace;
+	module.exports.getUniqueFileName = getUniqueFileName;
 	module.exports.getSignature = getSignature;
 	module.exports.getCommit = getCommit;
+	module.exports.postClone = postClone;
 
 	return express.Router()
 	.use(bodyParser.json())
-	.get(contextPath + '/workspace*', getClone)
-	.get(contextPath + '/file/:rootDir*', getClone)
-	.get(contextPath + '/file', getClone)
-	.put(contextPath + '/file*', putClone)
-	.delete(contextPath + '/file*', deleteClone)
+	.get(workspaceRoot + '*', getClone)
+	.get(fileRoot + '*', getClone)
+	.put(fileRoot + '*', putClone)
+	.delete(fileRoot + '*', deleteClone)
 	.post('*', postInit);
 
 function cloneJSON(base, location, giturl, parents, submodules) {
 	var result = {
-		"BranchLocation": contextPath + "/gitapi/branch" + location,
-		"CommitLocation": contextPath + "/gitapi/commit" + location,
-		"ConfigLocation": contextPath + "/gitapi/config/clone" + location,
-		"ContentLocation": location,
-		"DiffLocation": contextPath + "/gitapi/diff/Default" + location,
-		"HeadLocation": contextPath + "/gitapi/commit/HEAD" + location,
-		"IndexLocation": contextPath + "/gitapi/index" + location,
-		"Location": contextPath + "/gitapi/clone" + location,
+		"BranchLocation": gitRoot + "/branch" + location,
+		"CommitLocation": gitRoot + "/commit" + location,
+		"ConfigLocation": gitRoot + "/config/clone" + location,
+		"ContentLocation": contextPath + location,
+		"DiffLocation": gitRoot + "/diff/Default" + location,
+		"HeadLocation": gitRoot + "/commit/HEAD" + location,
+		"IndexLocation": gitRoot + "/index" + location,
+		"Location": gitRoot + "/clone" + location,
 		"Name": base,
 		"GitUrl": giturl,
 		"Children": submodules && submodules.length ? submodules : undefined,
 		"Parents": parents && parents.length ? parents : undefined,
-		"RemoteLocation": contextPath + "/gitapi/remote" + location,
-		"StashLocation": contextPath + "/gitapi/stash" + location,
-		"StatusLocation": contextPath + "/gitapi/status" + location,
-		"SubmoduleLocation": contextPath + "/gitapi/submodule" + location,
-		"TagLocation": contextPath + "/gitapi/tag" + location,
+		"RemoteLocation": gitRoot + "/remote" + location,
+		"StashLocation": gitRoot + "/stash" + location,
+		"StatusLocation": gitRoot + "/status" + location,
+		"SubmoduleLocation": gitRoot + "/submodule" + location,
+		"TagLocation": gitRoot + "/tag" + location,
 		"Type": "Clone"
 	};
 	if (giturl && isGithubURL(giturl)){
-		result["PullRequestLocation"] = contextPath + "/gitapi/pullRequest" + location;
+		result["PullRequestLocation"] = gitRoot + "/pullRequest" + location;
 	}
 	function isGithubURL(checkUrl){
 		var hostname = url.parse(checkUrl)["hostname"];
@@ -84,12 +91,16 @@ function cloneJSON(base, location, giturl, parents, submodules) {
 	return result;
 }
 	
-function getRepoByPath(filePath,workspaceDir) {
+function getRepoByPath(filePath, workspaceDir) {
 	while (!fs.existsSync(filePath)) {
 		filePath = path.dirname(filePath);
-		if (filePath.length <= workspaceDir) return Promise.reject(new Error("Forbidden"));
+		if (filePath.length <= workspaceDir.length) return Promise.reject(new Error("Forbidden"));
 	}
  	var ceiling = path.dirname(workspaceDir);
+	if (!fs.statSync(filePath).isDirectory()) {
+		// get the parent folder if pointing at a file
+		filePath = path.dirname(filePath);
+	}
 	return git.Repository.discover(filePath, 0, ceiling).then(function(buf) {
 		return git.Repository.open(buf.toString());
 	});
@@ -98,33 +109,35 @@ function getRepoByPath(filePath,workspaceDir) {
 function getRepo(req) {
 	var u = url.parse(req.url, true);
 	var restpath = u.pathname.split(fileRoot)[1] || "";
-	var filePath = path.join(req.user.workspaceDir, restpath);
-	return getRepoByPath(filePath,req.user.workspaceDir);
+	var file = fileUtil.getFile(req, restpath);
+	req.file = file;
+	return getRepoByPath(file.path, file.workspaceDir);
 }
 
-function getfileDir(repo ,req) {
+function getfileDir(repo, req) {
 	var fileDir;
-	if(repo.workdir().slice(0, -1).length === req.user.workspaceDir.length){
-		fileDir = api.join(fileRoot);
-	}else{
-		fileDir = api.join(fileRoot, repo.workdir().substring(req.user.workspaceDir.length + 1));
+	var file = req.workspaceDir ? req : req.file;
+	if (repo.workdir().slice(0, -1).length === file.workspaceDir.length) {
+		fileDir = api.join(fileRoot, file.workspaceId);
+	} else {
+		fileDir = api.join(fileRoot, file.workspaceId, repo.workdir().substring(file.workspaceDir.length + 1));
 	}
 	return fileDir;
 }
 
-function getfileDirPath(repo ,req) {
+function getfileDirPath(repo, req) {
 	var fileDirpath;
-	if(repo.workdir().slice(0, -1).length === req.user.workspaceDir.length){
+	var file = req.workspaceDir ? req : req.file;
+	if(repo.workdir().slice(0, -1).length === file.workspaceDir.length){
 		fileDirpath = path.join(fileRoot, path.sep);
 	}else{
-		fileDirpath = path.join(fileRoot, repo.workdir().substring(req.user.workspaceDir.length + 1));
+		fileDirpath = path.join(fileRoot, file.workspaceId, repo.workdir().substring(file.workspaceDir.length + 1));
 	}
 	return fileDirpath;
 }
 
 function getfileAbsolutePath(req) {
-	var fileAbsolutePath = path.join(req.user.workspaceDir, req.params["0"] || "");
-	return fileAbsolutePath;
+	return fileUtil.getFile(req, req.params["0"] || "").path;
 }
 
 function getfileRelativePath(repo, req) {
@@ -132,10 +145,6 @@ function getfileRelativePath(repo, req) {
 	return fileRelativePath;
 }
 
-function isWorkspace(req){
-	return !fs.existsSync(path.join(req.user.workspaceDir,'.git'));
-}
-
 function getCommit(repo, refOrCommit) {
 	return git.Reference.dwim(repo, refOrCommit).then(function(reference) {
 		if (reference.isTag()) {
@@ -156,17 +165,19 @@ function getCommit(repo, refOrCommit) {
 
 function getClone(req, res) {
 	getClones(req, res, function(repos) {
-		res.status(200).json({
+		writeResponse(200, res, null, {
 			"Children": repos,
 			"Type": "Clone"
-		});
+		}, true);
 	});
 }
 
 function getClones(req, res, callback) {
 	var repos = [];
 	
-	var rootDir = path.join(req.user.workspaceDir, req.params.rootDir || "");
+	var rest = req.params["0"].substring(1);
+	var file = fileUtil.getFile(req, rest);
+	var rootDir = file.path;
 		
 	checkDirectory(rootDir, function(err) {
 		if (err) return writeError(403, res, err.message);
@@ -174,7 +185,7 @@ function getClones(req, res, callback) {
 	});
 	
 	function pushRepo(repos, repo, base, location, url, parents, cb) {
-		Promise.all([url || getURL(repo), getSubmodules(repo, location, parents.slice(0).concat([contextPath + "/gitapi/clone" + location]))]).then(function(results) {
+		Promise.all([url || getURL(repo), getSubmodules(repo, location, parents.slice(0).concat([gitRoot + "/clone" + location]))]).then(function(results) {
 			var json = cloneJSON(base, location, results[0], parents, results[1]);
 			repos.push(json);
 			cb(json);
@@ -254,7 +265,7 @@ function getClones(req, res, callback) {
 			git.Repository.open(dir)
 			.then(function(repo) {
 				var base = path.basename(dir);
-				var location = getfileDir(repo ,req);
+				var location = getfileDir(repo, file);
 				pushRepo(repos, repo, base, location, null, [], function() { cb(); });
 	 		})
 			.catch(function() {
@@ -297,24 +308,53 @@ function configRepo(repo, username, email) {
 	});
 }
 
+function getClonePath(req) {
+	var workspacePath = req.body.Location;
+	var filePath = req.body.Path;
+	if (filePath && filePath.split("/").length < 3) {
+		filePath = undefined;
+	}
+	var file, rest;
+	var contextPathSegCount = (req.contextPath || "").split("/").length - 1;
+	if (filePath) {
+		rest = filePath.split("/").slice(2 + contextPathSegCount).join("/");
+		file = fileUtil.getFile(req, rest);
+	} else if (workspacePath) {
+		rest = workspacePath.split("/").slice(2 + contextPathSegCount).join("/");
+		file = fileUtil.getFile(req, rest);
+		if (file) {
+			var cloneName = req.body.Name;
+			var cloneUrl = req.body.GitUrl;
+			if (!cloneName && cloneUrl) {
+				if (cloneUrl.charAt(cloneUrl.length - 1) === "/") {
+					cloneUrl = cloneUrl.slice(0, -1);
+				}	
+				cloneName = cloneUrl.substring(cloneUrl.lastIndexOf("/") + 1).replace(".git", "");
+			}
+			if (!cloneName) return null;
+			file.path = getUniqueFileName(file.path, cloneName);
+		}
+	}
+	return file;
+}
+
 function postInit(req, res) {
 	if (req.body.GitUrl) {
 		postClone(req, res);
 	} else {
-		
-		if (req.body.Name === undefined) {
-			return writeError(400, res);
+		var file = getClonePath(req);
+		if (!file) {
+			return writeError(400, res, "Invalid parameters");
 		}
 		
-		var initDir = req.user.workspaceDir + '/' + req.body.Name;
 		var theRepo, index, author, committer;
 
-		fs.mkdir(initDir, function(err){
-			if (err) {
-				return writeError(409, res);
+		fs.mkdir(file.path, function(err) {
+			if (err && err.code !== "EEXIST") {
+				return writeError(400, res);
 			}
 
-			git.Repository.init(initDir, 0)
+			git.Repository.init(file.path, 0)
 			.then(function(repo) {
 				theRepo = repo;
 				return configRepo(repo, req.body.GitName, req.body.GitMail);
@@ -335,12 +375,11 @@ function postInit(req, res) {
 				return theRepo.createCommit("HEAD", author, committer, "Initial commit", oid, []);
 			})
 			.then(function() {
-				res.status(201).json({
-					"Location": contextPath + "/gitapi/clone"+ contextPath + "/file/" + req.body.Name
-				});
+				writeResponse(201, res, null, {
+					"Location": gitRoot + "/clone" + fileRoot + "/" + file.workspaceId + api.toURLPath(file.path.substring(file.workspaceDir.length))
+				}, true);
 			})
 			.catch(function(err){
-				console.log(err);
 				writeError(403, res);
 			});
 
@@ -354,7 +393,7 @@ function putClone(req, res) {
 	var tag = req.body.Tag;
 	var removeUntracked = req.body.RemoveUntracked;
 	if ((!paths || !paths.length) && !branch && !tag) {
-		return writeError(400, "Invalid parameters");
+		return writeError(400, res, "Invalid parameters");
 	}
 
 	var theRepo, theCommit;
@@ -400,8 +439,10 @@ function putClone(req, res) {
 		} else if (tag && typeof branch === "string") {
 			return git.Reference.lookup(theRepo, "refs/tags/" + tag)
 			.then(function(reference) {
-				return theRepo.getReferenceCommit(reference);
-			}).catch(function() {
+				return reference.peel(git.Object.TYPE.COMMIT);
+			}).then(function(oid) {
+				return theRepo.getCommit(oid);
+			}).catch(function(err) {
 				return theRepo.getTagByName(tag)
 				.then(function(tag) {
 					return tag.targetId();
@@ -416,7 +457,7 @@ function putClone(req, res) {
 				theCommit = commit;
 				if (branch) {
 					return git.Branch.create(theRepo, branch, commit, 0).then(function() {
-						return theRepo.checkoutBranch(branch, checkOptions);
+						return theRepo.checkoutBranch("refs/heads/" + branch, checkOptions);
 					});
 				}
 			 	return git.Checkout.tree(theRepo, commit, checkOptions).then(function() {
@@ -424,7 +465,7 @@ function putClone(req, res) {
 				});
 			});
 		}
-		return theRepo.checkoutBranch(branch, checkOptions);
+		return theRepo.checkoutBranch("refs/heads/" + branch, checkOptions);
 	})
 	.then(function(){
 		res.status(200).end();
@@ -435,8 +476,9 @@ function putClone(req, res) {
 }
 
 function deleteClone(req, res) {
-	var clonePath = req.params["0"];
-	rmdir(fileUtil.safeFilePath(req.user.workspaceDir, clonePath), function(err) {
+	var rest = req.params["0"];
+	var file = fileUtil.getFile(req, rest);
+	rmdir(file.path, function(err) {
 		if (err) return writeError(500, res, err);
 		res.status(200).end();
 	});
@@ -487,7 +529,7 @@ function foreachSubmodule(repo, operation, recursive) {
 	});
 }
 
-function getRemoteCallbacks(creds, task) {
+function getRemoteCallbacks(req, task) {
 	return {
 		certificateCheck: function() {
 			return 1; // Continues connection even if SSL certificate check fails. 
@@ -504,9 +546,7 @@ function getRemoteCallbacks(creds, task) {
 		 * @callback
 		 */
 		credentials: function(gitUrl, urlUsername) {
-			if (!creds.GitSshUsername && !creds.GitSshPrivateKey) {
-				return git.Cred.defaultNew();
-			}
+			var creds = req.body;
 			if (creds.GitSshPrivateKey) {
 				var privateKey = creds.GitSshPrivateKey;
 				var passphrase = creds.GitSshPassphrase;
@@ -517,24 +557,38 @@ function getRemoteCallbacks(creds, task) {
 					passphrase || ""
 				);
 			}
+
 			var username = creds.GitSshUsername || urlUsername;
 			var password = creds.GitSshPassword;
-			// clear username/password to avoid inifinite loop in nodegit
-			delete creds.GitSshUsername;
-			delete creds.GitSshPassword;
-			return git.Cred.userpassPlaintextNew(
-				username,
-				password || ""
-			);
+			if (username && password) {
+				/* clear username/password to avoid inifinite loop in nodegit */
+				delete creds.GitSshUsername;
+				delete creds.GitSshPassword;
+				return git.Cred.userpassPlaintextNew(
+					username,
+					password || ""
+				);
+			}
+
+			return new Promise(function(resolve, reject) {
+				credentialsProvider.getCredentials(gitUrl, req.user.username).then(
+					function(result) {
+						resolve(result);
+					},
+					function(error) {
+						resolve(git.Cred.defaultNew());
+					}
+				);
+			});
 		}
 	};
 }
 
 function handleRemoteError(task, err, cloneUrl) {
 	var fullCloneUrl;
-	if(cloneUrl.indexOf("://") !== -1){
+	if (cloneUrl.indexOf("://") !== -1){
 		fullCloneUrl = cloneUrl;
-	}else if(cloneUrl.indexOf("@") < cloneUrl.indexOf(":")){
+	} else if (cloneUrl.indexOf("@") < cloneUrl.indexOf(":")){
 		fullCloneUrl = "ssh://" + cloneUrl;
 	}
 	var u = url.parse(fullCloneUrl, true);
@@ -573,20 +627,17 @@ function getUniqueFileName(folder, file) {
 function postClone(req, res) {
 	var repo;
 	var cloneUrl = req.body.GitUrl;
-	if(cloneUrl.charAt(cloneUrl.length - 1) === "/"){
-		cloneUrl = cloneUrl.slice(0, -1);
-	}	
-	var dirName = cloneUrl.substring(cloneUrl.lastIndexOf("/") + 1).replace(".git", "");
-	var folder = req.user.workspaceDir;
-	if (req.body.Path) {
-		folder = path.join(folder, req.body.Path.substring(fileRoot.length));
+	
+	var file = getClonePath(req);
+	if (!file) {
+		return writeError(400, res, "Invalid parameters");
 	}
 	
 	var task = new tasks.Task(res, false, true, 0, true);
 	
-	git.Clone.clone(cloneUrl, getUniqueFileName(folder, dirName), {
+	return git.Clone.clone(cloneUrl, file.path, {
 		fetchOpts: {
-			callbacks: getRemoteCallbacks(req.body, task)
+			callbacks: getRemoteCallbacks(req, task)
 		}
 	})
 	.then(function(_repo) {
@@ -594,7 +645,8 @@ function postClone(req, res) {
 		return configRepo(repo, req.body.GitName, req.body.GitMail);
 	})
 	.then(function() {
-		if (req.body.cloneSubmodules) {
+		// default to true if parameter not set
+		if (req.body.cloneSubmodules === undefined || req.body.cloneSubmodules === null || req.body.cloneSubmodules) {
 			return foreachSubmodule(repo, "update", true);
 		}
 	})
@@ -604,7 +656,7 @@ function postClone(req, res) {
 			Code: 0,
 			DetailedMessage: "OK",
 			JsonData: {
-				Location: contextPath + "/gitapi/clone" + fileRoot + "/" + dirName
+				Location: gitRoot + "/clone" + fileRoot + "/" + file.workspaceId + api.toURLPath(file.path.substring(file.workspaceDir.length))
 			},
 			Message: "OK",
 			Severity: "Ok"
